//
//  GaugeView.m
//  Gyroscope
//
//  Created by Phidgets on 2015-07-24.
//  Copyright (c) 2015 Phidgets. All rights reserved.
//

#import "AccelGaugeView.h"
#define HIGHPRECISION_1043_1044 1
static inline CGRect
convertToCGRect(NSRect rect)
{
    return CGRectMake(rect.origin.x, rect.origin.y,
                      rect.size.width, rect.size.height);
}
@implementation AccelGaugeView

- (id)initWithFrame:(NSRect)frameRect
{
    if(self = [super initWithFrame:frameRect])
    {
        img = [NSImage imageNamed:@"needle"];
    }
    
    return self;
}

-(void)setAngle:(double)deg{
    deg = -16.875*deg + 270;
    angle = deg * M_PI/180;
    [self setNeedsDisplay:YES];
}

-(void)setType:(int)accelType{
    type = accelType;
}

- (NSImage*)rotateImage:(NSImage*)orig byDegrees:(float)deg{
    NSImage *rotated = [[NSImage alloc] initWithSize:[orig size]];
    [rotated lockFocus];
    
    //Need to translate the image so it's centered at the origin before rotating
    NSAffineTransform *transform = [NSAffineTransform transform];
    [transform translateXBy:[orig size].width/2.0 yBy:[orig size].height/2.0];
    [transform rotateByRadians:deg];
    [transform translateXBy:-[orig size].width/2.0 yBy:-[orig size].height/2.0];
    [transform concat];
    
    [orig drawAtPoint:NSZeroPoint fromRect:NSZeroRect operation:NSCompositingOperationCopy fraction:1.0];
    [rotated unlockFocus];
    return rotated;
}

- (void)drawRect:(NSRect)re {
    [super drawRect:re];
    
    float w, h, r;
    CGRect rect;
    CGContextRef context;
    
    rect = convertToCGRect(re);
    context = [[NSGraphicsContext currentContext] graphicsPort];
    
    w = CGRectGetWidth(rect);
    h = CGRectGetHeight(rect);
    r = (w < h) ? w/2.5 : h/2.5;

    //Draw the image
    [[NSGraphicsContext currentContext]
     setImageInterpolation: NSImageInterpolationHigh];
    
    NSSize viewSize  = [self bounds].size;
    double newWidth = (int)([img size].width * (h / [img size].height) * 0.8);
    double newHeight = (int)(h * 0.8 );
    NSSize imageSize = { newWidth, newHeight };
    
    NSPoint viewCenter;
    viewCenter.x = viewSize.width  * 0.50;
    viewCenter.y = viewSize.height * 0.50;
    
    NSPoint imageOrigin = viewCenter;
    imageOrigin.x -= imageSize.width  * 0.50;
    imageOrigin.y -= imageSize.height * 0.50;
    
    NSRect destRect;
    destRect.origin = imageOrigin;
    destRect.size = imageSize;
    
    NSImage *rotatedImage = [self rotateImage:[img copy] byDegrees:angle];
    [rotatedImage drawInRect:destRect fromRect:NSZeroRect
                   operation: NSCompositingOperationSourceOver
                    fraction: 1.0];
    
    /* Draw a cricle */
    CGContextBeginPath(context);
    CGContextAddArc(context, CGRectGetMidX(rect), CGRectGetMidY(rect), r, 225*M_PI/180, 315*M_PI/180, true);
    CGContextStrokePath(context);
    
    if(type == HIGHPRECISION_1043_1044){
        CGContextSetLineWidth(context, 16);
        CGContextBeginPath(context);
        CGContextAddArc(context, CGRectGetMidX(rect), CGRectGetMidY(rect), r-8, 225*M_PI/180, 123.75*M_PI/180, true);
        CGContextSetRGBStrokeColor(context, 1, 0, 0, 0.4);
        CGContextStrokePath(context);
        CGContextAddArc(context, CGRectGetMidX(rect), CGRectGetMidY(rect), r-8, 56.25*M_PI/180, 315*M_PI/180, true);
        CGContextStrokePath(context);
    }
    //Ticks around the compass circle and numbering
    CGContextSetRGBStrokeColor(context, 0, 0, 0, 1);
    
    NSDictionary *textAttributes = @{NSFontAttributeName: [NSFont systemFontOfSize:h/20]};
    int j = -8, xOff = 0, yOff = 0;
    for(double i = 135 ; i <= 405; i += 1.6875)
    {
        int tickSize = 0;
        if(fmod((i - 135),16.875) == 0){
            //draw labels
            tickSize = 16;
            CGContextSetLineWidth(context, 2);
            if(j < -6){
                xOff = -12;
                yOff = -10;
            }
            else if (j < -3){
                xOff = -15;
                yOff = -5;
            }
            else if (j < 0){
                xOff = -12;
                yOff = -1;
            }
            else if (j == 0){
                xOff = -3;
                yOff = 2;
            }
            else if (j < 4){
                xOff = 3;
                yOff = 0;
            }
            else if (j < 6){
                xOff = 3;
                yOff = -4;
            }
            else{
                xOff = 3;
                yOff = -8;
            }
            [[NSString stringWithFormat:@"%d",j++] drawAtPoint:NSMakePoint(r * cos(i * (M_PI / 180.0)) + CGRectGetMidX(rect) + xOff,  -r  * sin(i * (M_PI / 180.0)) + CGRectGetMidY(rect) + yOff) withAttributes:textAttributes];
        }
        else if(fmod((i - 135),8.4375) == 0){
            tickSize = 8;
            CGContextSetLineWidth(context, 1);
        }
        else{
            tickSize = 4;
            CGContextSetLineWidth(context, 0.5);
        }
        CGContextSetRGBStrokeColor(context, 0, 0, 0, 1);
        CGContextMoveToPoint(context, r * cos(i * (M_PI / 180.0)) + CGRectGetMidX(rect),
                             -r  * sin(i * (M_PI / 180.0)) + CGRectGetMidY(rect));
        CGContextAddLineToPoint(context, (r - tickSize) * cos(i * (M_PI / 180.0)) + CGRectGetMidX(rect),
                                -(r - tickSize) * sin(i * (M_PI / 180.0)) + CGRectGetMidY(rect));
        CGContextStrokePath(context);
       
        
        
    }
}

@end




















